This program aims to be a drop-in replacement
for [Harpy's Demultiplex] pipeline step.

Advantages include:

- Disk space: no intermediate files are written on disk.
  - Inputs: `{I1,I2,R1,R2}.fq.gz` + `*.schema` + `BC_{A,B,C,D}.txt`.
  - (Direct) Outputs: `samples/{Sample1,Sample2,..}.fq.gz`

- Per-CPU performance:
  - Edit distances are memoized
    to avoid repeated Hamming/Levenshtein calculations.
  - Output sample files are zipped using modern [`flate2`] crate
    with [`zlib-ng`] backend.

- Multithreaded performance:
  - Input unzipping is parallelized over input files.
  - Output zipping is parallelized over output files
    among a configurable number of "writer" workers.
  - Backpressure: reader threads are faster than writer threads
    but they block on the latter
    so the RAM does not get filled with parsed input files.

- Control:
  - Modern, configurable, tested and documented [Rust] codebase.
  - If included within Harpy pipeline,
    I commit to maintain this program and evolve it
    according to Harpy's needs
    as part of my [permanent job].

### Use

Building only requires a [nightly rust toolchain]
and [`git`] to retrieve the source code.

```sh
$ git clone https://gitlab.mbb.cnrs.fr/ibonnici/dmox.git
$ cd dmox
$ cargo build --release
$ cargo test --release
$ ./target/release/dmox --help # Standalone, executable binary file.
```

[Harpy's Demultiplex]: https://pdimens.github.io/harpy/workflows/demultiplex/
[`flate2`]: https://docs.rs/flate2/latest/flate2/
[`zlib-ng`]: https://github.com/zlib-ng/zlib-ng
[Rust]: https://www.rust-lang.org
[permanent job]: https://isem-evolution.fr/en/membre/bonnici/
[`git`]: https://git-scm.com/
[nightly rust toolchain]: https://rust-lang.github.io/rustup/concepts/channels.html#working-with-nightly-rust
