// Parameters to the procedure.

use std::{
    fmt::{self, Display},
    path::PathBuf,
};

pub struct Config {
    pub module_size: usize,
    pub n_modules: u64,
    pub output_compression_level: u32,
    pub inputs: [PathBuf; 4],     // [I1, I2, R1, R2]
    pub references: [PathBuf; 4], // [A, B, C, D]
    pub distance: Distance,
    pub max_distance: Option<usize>,
    pub schema: PathBuf,
    pub clear: PathBuf,
    pub unclear: PathBuf,
    pub sample: Box<SamplePath>,
    pub readers_channel_capacity: usize,
    pub n_writers: usize,
    pub max_queued_blocks: usize,
    pub writers_capacity: usize,
}

#[derive(clap::ValueEnum, Clone, Copy)]
pub enum Distance {
    Hamming,
    Levenshtein,
}

type SamplePath = dyn Fn(&str, &str) -> PathBuf; // { (sample id, infix) -> path }

// Useful for allocation-free levenshtein distance.
pub const MAX_MODULE_SIZE: usize = 6;

// (for clap)
impl Display for Distance {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        use Distance as D;
        let d = match self {
            D::Hamming => "hamming",
            D::Levenshtein => "levenshtein",
        };
        write!(f, "{d}")
    }
}
